
package pg.ntnu.eti.ibm.master.thesis.paralysis.server;

import java.util.ArrayList;
import org.opencv.core.Core;
import static org.opencv.core.Core.NORM_MINMAX;
import static org.opencv.core.Core.getOptimalDFTSize;
import static org.opencv.core.Core.merge;
import static org.opencv.core.Core.normalize;
import static org.opencv.core.Core.split;
import static org.opencv.core.CvType.CV_32FC3;
import static org.opencv.core.CvType.CV_8UC3;
import org.opencv.core.Mat;
import org.opencv.core.Rect;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;
import static org.opencv.imgproc.Imgproc.pyrDown;
import static org.opencv.imgproc.Imgproc.pyrUp;

/**
 *
 * @author akwasnie
 */
public class ColorMagnifier {

    private static ArrayList<Mat> pyramid = new ArrayList<>();
    private static ArrayList<Mat> lowpass1 = new ArrayList<>();
    private static ArrayList<Mat> lowpass2 = new ArrayList<>();
    private static ArrayList<Mat> filtered = new ArrayList<>();
    private static ArrayList<Mat> downsampled = new ArrayList<>();
    private static int levels = 3;
    private static Mat dst;

    public static ArrayList<Mat> change(ArrayList<Mat> facesArray) {
        pyramid.clear();
        filtered.clear();
        downsampled.clear();
        ArrayList<Mat> al = new ArrayList<Mat>();
        ArrayList<Mat> alRet = new ArrayList<Mat>();
        Mat im = new Mat();
        for (Mat facesArray1 : facesArray) {
            im = facesArray1;
            if (!im.empty()) {
                im.convertTo(im, CV_32FC3);
                al.add(im.clone());
                buildGaussPyrFromImg(im, levels);
                Mat downSampledFrame = pyramid.get(levels - 1);
                downsampled.add(downSampledFrame);
            }
        }
        concate(downsampled);

        Mat dsf = amplifyGaussian(idealFilter(dst));

        filtered = deConcat(dsf);
        Mat d = new Mat();
        if (al.size() > 0) {
            for (int i = 0; i < al.size(); i++) {

                d = buildImgFromGaussPyr(filtered.get(i), al.get(i).size());
                Mat b = new Mat();
                Imgproc.resize(al.get(i), b, d.size());
                Core.add(b, d, d);
                ArrayList<Mat> ch = new ArrayList<>();
                split(d, ch);
                Core.MinMaxLocResult minMaxLoc = Core.minMaxLoc(ch.get(0));
                d.convertTo(d, CV_8UC3, 255.0 / (minMaxLoc.maxVal - minMaxLoc.minVal), -2 * 255.0 / (minMaxLoc.maxVal - minMaxLoc.minVal));
                alRet.add(d);
            }
        }

        return alRet;
    }

    private static Mat buildImgFromGaussPyr(Mat pyr, Size size) {
        Mat r = new Mat();
        Mat currentLevel = pyr.clone();
        for (int i = 0; i < levels; ++i) {
            Mat up = new Mat();
            pyrUp(currentLevel, up);
            currentLevel = up;
        }
        currentLevel.copyTo(r);
        return r;
    }

    private static void buildGaussPyrFromImg(Mat image, int levels) {

        pyramid.clear();
        Mat currentImg = image;
        for (int l = 0; l < levels; l++) {
            Mat down = new Mat();
            pyrDown(currentImg, down);
            pyramid.add(down);
            currentImg = down;
        }
    }

    private static Mat idealFilter(Mat src) {
        Mat a = new Mat();
        ArrayList<Mat> channels = new ArrayList<>();
        split(src.clone(), channels);
        for (int curChannel = 0; curChannel < channels.size(); ++curChannel) {
            Mat current = channels.get(curChannel);
            Mat tempImg = new Mat();
            int width = current.cols();
            int height = getOptimalDFTSize(current.rows());
            Imgproc.copyMakeBorder(current, tempImg, 0, height - current.rows(), 0, width - current.cols(), Imgproc.BORDER_CONSTANT, Scalar.all(0));
            Core.dft(tempImg, tempImg, Core.DFT_ROWS | Core.DFT_SCALE, Core.DFT_ROWS | Core.DFT_SCALE);
            Mat filter = tempImg.clone();
            filter = createIdealBandpassFilter(filter.clone(), 0.05, 0.8, 25);
            Core.mulSpectrums(tempImg, filter, tempImg, Core.DFT_ROWS);
            Core.idft(tempImg, tempImg, Core.DFT_ROWS | Core.DFT_SCALE, Core.DFT_ROWS | Core.DFT_SCALE);
            Rect r = new Rect(0, 0, current.cols(), current.rows());
            Mat roi = new Mat(tempImg, r);
            roi.copyTo(channels.get(curChannel));
        }
        merge(channels, a);
        normalize(a, a, 0, 1, NORM_MINMAX);
        return a;
    }

    private static Mat amplifyGaussian(Mat im) {
        Scalar s = new Scalar(2);
        Mat m = im.clone();
        Core.multiply(m, s, m);
        return m;
    }

    private static Mat createIdealBandpassFilter(Mat filter, double cutoffLo, double cutoffHi, int framerate) {

        float width = filter.cols();
        float height = filter.rows();
        double fl = 2 * cutoffLo * width / framerate;
        double fh = 2 * cutoffHi * width / framerate;
        double response;
        for (int y = 0; y < height; ++y) {
            for (int x = 0; x < width; ++x) {
                if (x >= fl && x <= fh) {
                    response = 1.0;
                } else {
                    response = 0.0;
                }
                filter.put(y, x, response);

            }
        }
        return filter;
    }

    private static Mat concate(ArrayList<Mat> frames) {

        Size frameSize = frames.get(0).size();

        Mat temp = new Mat((int) (frameSize.width * frameSize.height), levels - 1, CV_32FC3);
        for (int i = 0; i < levels - 1; ++i) {
            Mat input = frames.get(i);
            Mat reshaped = input.reshape(3, input.cols() * input.rows()).clone();
            Mat line = temp.col(i);
            reshaped.copyTo(line);
        }
        temp.copyTo(dst);
        return dst;
    }

    private static ArrayList<Mat> deConcat(Mat src) {
        Size frameSize = downsampled.get(0).size();
        ArrayList<Mat> a = new ArrayList<>();
        for (int i = 0; i < levels - 1; ++i) { 
            Mat line = src.col(i).clone();
            Mat reshaped = line.reshape(3, (int) frameSize.height).clone();
            a.add(reshaped);
        }
        return a;
    }
}
