
package pg.ntnu.eti.ibm.master.thesis.paralysis.server;

/**
 *
 * @author akwasnie
 */


import java.awt.*;
import java.io.File;
import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;
import org.opencv.core.Mat;
import org.opencv.core.MatOfByte;
import org.opencv.highgui.Highgui;

public class ColorsCount {
    private static int colorBucketWidth = 20;

    public static int GetColorBucketWidth() {
        return colorBucketWidth;
    }

    public static void SetColorBucketWidth(int colorBucketWidth) {
        if(colorBucketWidth < 1)
            ColorsCount.colorBucketWidth = 1;
        else if(colorBucketWidth > 255)
            ColorsCount.colorBucketWidth = 255;
        else
            ColorsCount.colorBucketWidth = colorBucketWidth;
    }

    public static Map<Integer, Integer> GetCount(String sourcePath){
        try {
            File sourceFile = new File(sourcePath);
            return ColorsCount.GetCount(sourceFile);
        } catch (NullPointerException e) {
            e.printStackTrace();
            return new HashMap<>();
        }
    }

    public static Map<Integer, Integer> GetCount(File sourceFile){
        try {
            BufferedImage sourceImage = ImageIO.read(sourceFile);
            return ColorsCount.GetCount(sourceImage);
        } catch (IOException e) {
            e.printStackTrace();
            return new HashMap<>();
        }
    }

    public static Map<Integer, Integer> GetCount(BufferedImage sourceImage){
        Map<Integer, Integer> colors= new HashMap<>();
        int value;
        for(int x=0; x<sourceImage.getWidth(); x++){
            for(int y=0; y<sourceImage.getHeight(); y++){
                Color color = new Color(sourceImage.getRGB(x,y));
                Color colorBucket = ColorsCount.GetColorBucket(color);
                if(colors.containsKey(colorBucket.getRGB())){
                    value = colors.get(colorBucket.getRGB())+1;
                }
                else{
                    value = 1;
                }
                colors.put(colorBucket.getRGB(), value);
            }
        }
        return colors;
    }

    public static Color GetColorBucket(Color color){
        return ColorsCount.GetColorBucket(color.getRed(), color.getGreen(), color.getBlue());
    }

    public static Color GetColorBucket(int red, int green, int blue){
        red = (red / ColorsCount.GetColorBucketWidth()) * ColorsCount.GetColorBucketWidth();
        green = (green / ColorsCount.GetColorBucketWidth()) * ColorsCount.GetColorBucketWidth();
        blue = (blue/ ColorsCount.GetColorBucketWidth()) * ColorsCount.GetColorBucketWidth();
        return new Color(red,green,blue);
    }

    public static Color PrintColorsMap(Map<Integer, Integer> colors){
        int max=0;
        int keyRet = 0;
        for(Integer key : colors.keySet()){
            Integer value = colors.get(key);
           // System.out.println(new Color(key) + " : " + value);
            if (value>max && !new Color(key).equals(new Color(0, 0, 0))){
                max=value;
                keyRet=key;
            }
        }
            return new Color(keyRet);
        
    }

    static BufferedImage convert(Mat mat) throws IOException {
        
        MatOfByte bytemat = new MatOfByte();
        Highgui.imencode(".jpg", mat, bytemat);
        byte[] bytes = bytemat.toArray();
        InputStream in = new ByteArrayInputStream(bytes);
        BufferedImage read = ImageIO.read(in);
        in.close();

        return read;
    }
    

}

