
package pg.ntnu.eti.ibm.master.thesis.paralysis.server;

import static java.lang.Math.sqrt;
import java.util.ArrayList;
import org.opencv.core.Core;
import static org.opencv.core.Core.merge;
import static org.opencv.core.Core.split;
import static org.opencv.core.CvType.CV_32FC3;
import static org.opencv.core.CvType.CV_8UC3;
import org.opencv.core.Mat;
import org.opencv.core.Scalar;
import org.opencv.core.Size;
import org.opencv.imgproc.Imgproc;
import static org.opencv.imgproc.Imgproc.pyrDown;
import static org.opencv.imgproc.Imgproc.pyrUp;

/**
 *
 * @author akwasnie
 */
public class MotionMagnifier {

    private static ArrayList<Mat> lowpass1 = new ArrayList<>();
    private static ArrayList<Mat> lowpass2 = new ArrayList<>();
    private static ArrayList<Mat> filtered = new ArrayList<>();
    private static ArrayList<Mat> pyramid = new ArrayList<>();
    private static int curLevel;
    private static int levels = 3;
    private static double delta;
    private static double exaggeration_factor;
    private static double lambda;

    public static ArrayList<Mat> move(ArrayList<Mat> facesArrayProc) {

        ArrayList<Mat> a = new ArrayList<>();

        for (int i = 0; i < facesArrayProc.size(); i++) {
            Mat m = facesArrayProc.get(i);
            m.convertTo(m, CV_32FC3, 1.0 / 255.0f);
            Mat s = m.clone();
            buildLaplacianPiramid(m);
            if (i == 0) { 
                lowpass1 = pyramid;
                lowpass2 = pyramid;
                filtered = pyramid;
            } else {
                for (int j = 0; j < levels; ++j) {
                    curLevel = j;
                    temporalFilter(pyramid.get(j));
                }
                Size filterSize = filtered.get(0).size();
                double w = filterSize.width;
                double h = filterSize.height;
                delta = 80 / 8.0 / (1.0 + 10);
                exaggeration_factor = 2.0;
                lambda = sqrt(w * w + h * h) / 3; 
                for (int k = levels; k >= 0; k--) {
                    curLevel = k;
                    Mat f = amplify(filtered.get(k));
                    filtered.set(k, f);
                    lambda /= 2.0;
                }
            }
            Mat motion = reconImgFromLaplacianPyramid(filtered);
            motion = attenuate(motion.clone());
            Core.add(s, motion, s);
            Mat output = s.clone();
            output.convertTo(output, CV_8UC3, 255.0, 1.0 / 255.0);
            a.add(output);
        }
        return a;
    }

    private static void buildLaplacianPiramid(Mat img) {
        pyramid.clear();
        Mat currentImg = img;
        for (int l = 0; l < levels; l++) {
            Mat down = new Mat();
            Mat up = new Mat();
            pyrDown(currentImg, down);
            pyrUp(down, up, currentImg.size());
            Mat lap = new Mat();
            Core.subtract(currentImg, up, lap);
            pyramid.add(lap);
            currentImg = down;
        }
        pyramid.add(currentImg);
    }


    private static void temporalFilter(Mat src) {
        Mat a = new Mat();
        Mat temp1 = new Mat();
        Mat temp2 = new Mat();
        Mat t1 = new Mat();
        Mat t2 = new Mat();
        Mat t11 = new Mat();
        Mat t22 = new Mat();
        Scalar s = new Scalar(0.8);
        Scalar s1 = new Scalar(0.2);
        Scalar s2 = new Scalar(0.05);
        Scalar s3 = new Scalar(0.95);
        Core.multiply(src, s, t1);
        Core.multiply(src, s2, t2);
        Core.multiply(lowpass1.get(curLevel), s1, t11);
        Core.multiply(lowpass2.get(curLevel), s3, t22);
        Core.add(t1, t11, temp1);
        Core.add(t2, t22, temp2);
        lowpass1.set(curLevel, temp1);
        lowpass2.set(curLevel, temp2);
        Core.subtract(lowpass1.get(curLevel), lowpass2.get(curLevel), a);
        filtered.add(a);
    }

    private static Mat amplify(Mat src) {
        Mat d = new Mat();
        double currAlpha = lambda / delta / 7;
        currAlpha *= exaggeration_factor;
        if (curLevel == levels || curLevel == 0) 
        {
            Core.multiply(src, new Scalar(0), d);
        } else {
            Core.multiply(src, new Scalar(Math.min(2, currAlpha)), d);
        }
        return d;

    }

    private static Mat reconImgFromLaplacianPyramid(ArrayList<Mat> filtered) {
        Mat currentImg = filtered.get(levels - 1);
        Mat d = new Mat();
        for (int l = levels - 1; l >= 0; l--) {
            Mat up = new Mat();
            pyrUp(currentImg, up);
            Imgproc.resize(up, up, filtered.get(l).size());
            Core.add(up, filtered.get(l), currentImg);
        }
        d = currentImg.clone();
        return d;
    }

    private static Mat attenuate(Mat src) {
        ArrayList<Mat> planes = new ArrayList<>();
        Mat d = new Mat();
        Mat a = new Mat();
        Mat b = new Mat();
        split(src, planes);
        Core.multiply(planes.get(1), new Scalar(2), a);
        Core.multiply(planes.get(1), new Scalar(5), b);
        planes.set(1, a);
        planes.set(2, b);
        merge(planes, d);
        return d;
    }
}
